//============================================================================
// Name        : mandelbrot.cpp
// Author      : Gianluca Ghettini
// Version     : 1.0
// Copyright   : 
// Description : Realtime zoomer dell'insieme di Mandelbrot
//============================================================================

#include <iostream>
#include <math.h>
#include <cstdio>
#include "PixelToaster.h"

using namespace std;
using namespace PixelToaster;

// risoluzione
#define SCREEN_W 200
#define SCREEN_H 200
#define RESOLUTION 1

TrueColorPixel *frame;


void draw(double minx,double maxx,double miny,double maxy,double max_iter,double max_mod,double scala,double xa,double xb)
{
	double dx,dy,ca,cb,za,zb,old_za,mul_za,mul_zb,r;
	int x,y,col,offset,i,j;
	za = 0;
	zb = 0;      
	ca = 0;
	cb = 0;
	dx = (maxx - minx) / SCREEN_W;    
	dy = (maxy - miny) / SCREEN_H;
	for(y=0;y<SCREEN_H;y++)
	{
		cb = (miny + y*dy)*scala - xb;
		for(x=0;x<SCREEN_W;x++)
		{
			ca = (minx + x*dx)*scala + xa;     
			col = 0;
			za = ca;
			zb = cb;
			
			while(1)
			{
				old_za = za;
				mul_za = za*za;
				mul_zb = zb*zb;
				za = mul_za - mul_zb + ca;
				zb = old_za*zb*2 + cb;
				r = mul_za + mul_zb;
				col++;
				if (r>max_mod) 
				{
					break;
				}
				if (col>max_iter)
				{
					col = 0;
					break;
				}
			}
			
			offset = x + SCREEN_W * y;
			frame[offset].r = col<<1;
			frame[offset].g = col<<1;
			frame[offset].b = col<<2;
		}
	}              
}



int main()
{
	// init schermo
	Display display("Mandelbrot", SCREEN_W, SCREEN_H, Output::Windowed);
	display.open();


	frame = new TrueColorPixel[SCREEN_W * SCREEN_H];
	
	double minx,maxx,miny,maxy,max_mod,max_iter,vzoom,xa,xb,scala,q;
	
	vzoom = 1.1;	/* velocita di zoom */
	xa = -0.376900000058;	/* punto complesso di zooming */
	xb = -0.638691000021;
	q = 10;	/* fattore di correzione per max_iter */
	max_iter = 40;	/* numero massimo di iterazioni */
	max_mod = 4;	/* distanza del modulo */
	minx = -2;	/* finestra di partenza */
	maxx = 1.25;
	miny = -1.25;
	maxy = 1.25;
	scala = 1;	/* fattore di scala: inizialmente 1:1 */
	

	
	while(1)
	{
		draw(minx,maxx,miny,maxy,max_iter,max_mod,scala,xa,xb);
		display.update(frame);
		scala = scala / vzoom;
		max_iter = max_iter + q;
	}
}
